
    
local OldModBlueprints = ModBlueprints
function ModBlueprints(all_blueprints)
    OldModBlueprints(all_blueprints)

    for uID, uBP in all_blueprints.Unit do
        -- if we dont have categories or LODs we can't calculate the new LOD.
        if not uBP.Categories or not uBP.Display.Mesh.LODs then continue end
        CalculateNewLod(uBP)
    end
end

-- LODoptions: Changing this in multiplayer will desync the game! Default is 2
-- 0 diabled
-- 1 low / Units will be hide by size if zoomed out. Using High and Low resolution textures and models
-- 2 high / Units hide by size and ony use highres textures and models.
-- 3 full / Units will never hide, and only use highres textures and models.

local LODoptions = 1




function CalculateNewLod(uBP)
    if LODoptions == 0 then return end
    -- count how many LOD entries exist (max 2)
    local LODsCount = table.getn(uBP.Display.Mesh.LODs)
    -- if we don't have LOD settings, return
    if LODsCount <= 0 then return end
    -- If we use only higrestextures we move LODCutoff from LOD-2 to LOD-1 and delete the 2nd LOD entry
    if LODoptions == 2 or LODoptions == 3 then
        LODsCount = 1 
        -- do we have LODCutoff in 2nd LOD array ? 
        if uBP.Display.Mesh.LODs[2].LODCutoff then
            -- copy LODCutoff from lowres to highres LOD
            uBP.Display.Mesh.LODs[1].LODCutoff = uBP.Display.Mesh.LODs[2].LODCutoff
        end
        -- delete the 2nd LOD entry
        uBP.Display.Mesh.LODs[2] = nil
    end
    -- if we dont hide units while zooming out, we set a fixed LODCutoff and return
    if LODoptions == 3 then
        uBP.Display.Mesh.LODs[1].LODCutoff = 1000
        return
    end
    -- copy categories into local variable for faster access
    local Categories = {}
    for _, cat in uBP.Categories do
        Categories[cat] = true
    end
    -- loop over LOD's array
    for lodNum, lod in uBP.Display.Mesh.LODs do
        -- we only check the last LOD entry inside the LOD array, and only if we have SelectionSizeX + Z
        if lodNum ~= LODsCount or not uBP.SelectionSizeX or not uBP.SelectionSizeZ then continue end
        -- calculate unit/building surface by SelectionSize or HitBoxSize, whatever is bigger
        local SelectionSize = uBP.SelectionSizeX * uBP.SelectionSizeZ
        local HitBoxSize = uBP.SizeX / 1.4  * uBP.SizeZ / 1.4
        if SelectionSize > HitBoxSize then
            UnitLodSize = SelectionSize
        else
            UnitLodSize = HitBoxSize
        end
        -- make Experimental bigger, if it's not already as big as an factory. So we see experimentals a bit longer if zoomed out
        if (Categories.EXPERIMENTAL or uBP.StrategicIconName == 'icon_experimental_generic') and UnitLodSize < 5.5 then -- Experimentals smaller then factories
            UnitLodSize = UnitLodSize * 1.8
        end
        -- mq use 125 as offset, so we start hiding units after strategic icons are shown.
        local mq = math.floor(UnitLodSize *35)+125 -- to compare: LandFactoryLODsize x*y = 10,56.
        -- dont hide units until we display the strategic icon
        if mq < 160 then
            mq = 160
        end
        -- stop displaying land units at LOD 600 (LANDFactory has ~ 500)
        if mq > 600 and not Categories.AIR and not Categories.EXPERIMENTAL then
            mq = 600
        end
        -- stop displaying air units a little bit later then land units
        if mq > 700 and Categories.AIR and not Categories.EXPERIMENTAL then
            mq = 700
        end
        -- stop displaying all units over LOD 1000. (almost zoomed out max)
        if mq > 1000 then
            mq = 1000
        end
        -- display strategic icons from LOD 160 on, so they will hide smaller units.
        uBP.Display.Mesh.IconFadeInZoom = 160
        -- Set the new LODCutoff inside blueprint LOD-1 or LOD-2
        uBP.Display.Mesh.LODs[lodNum].LODCutoff = mq
        -- if we have 2 LOD settings (hig and low res), set LODCutoff from LOD-1(HighresLOD) to 1/2 of LOD-2 (mq).
        if lodNum == 2 then
            local HighresLOD = mq/2
            -- dont hide units until we display the strategic icon
            if HighresLOD < 160 then
                HighresLOD = 160
            end
            -- Set the new LODCutoff inside blueprint LOD-1
            uBP.Display.Mesh.LODs[1].LODCutoff = HighresLOD
        end
    end
end

